#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <iostream>
#include "modbus.h" //modbus动态库文件

int main(int argc, char *argv[])
{
   // 1.read ttyS number
   if (argc < 2)
   {
      fprintf(stderr, "Usage: %s <ttyS_device>\n", argv[0]);
      return -1;
   }
   char tty_device[20];
   snprintf(tty_device, sizeof(tty_device), "/dev/%s", argv[1]);

   // 创建modbus对象
   modbus_t *modbusContext = nullptr;

   // 初始化主站的相关寄存器
   modbus_mapping_t *mb_mapping = NULL;
   mb_mapping = modbus_mapping_new(10, 10, 10, 10);
   if (mb_mapping == NULL)
   {
      fprintf(stderr, "Failed to allocate the mapping: %s\n", modbus_strerror(errno));
      modbus_free(modbusContext);
      return -1;
   }

   // 初始化 - 设备号、波特率、校验位、数据位、停止位
   modbusContext = modbus_new_rtu(tty_device, 115200, 'N', 8, 1);
   if (modbusContext == nullptr)
   {
      std::cerr << "Failed to create Modbus context" << std::endl;
      return 1;
   }

   // 从机 ID
   const int slaveId = 1;
   modbus_set_debug(modbusContext, 1);       // 设置1可看到调试信息
   modbus_set_slave(modbusContext, slaveId); // 设置slave ID

   // 打开 Modbus 连接
   if (modbus_connect(modbusContext) == -1)
   {
      std::cerr << "Modbus connection failed: " << modbus_strerror(errno) << std::endl;
      modbus_free(modbusContext);
      return 1;
   }

   std::cout << "Choose Modbus mode:" << std::endl;
   std::cout << "1. Modbus-RTU 主机模式" << std::endl;
   std::cout << "2. Modbus-RTU 从机模式" << std::endl;
   int choice;
   std::cin >> choice;

   uint16_t data = 0;
   int rc = -1;

   int64_t slave_ok = 0, slave_erro = 0;
   int64_t poll_ok = 0, poll_erro = 0;
   int64_t cyc_cont = 0;
   while (1)
   {
      cyc_cont++;
      switch (choice)
      {
      case 1:
      {
         std::cout << "-----------主机模式----------" << std::endl;
         uint16_t read_buffer_16[1024]; // 定义read_buffer_16
         // 0x03：读保持寄存器: [01][03][00][00][00][0A][C5][CD]
         // recive:<01><03><14><00><00><00><00><00><00><00><00><00><00><00><00><00><00><00><00><00><00><00><00><A3><67>
         rc = modbus_read_registers(modbusContext, 0, 10, read_buffer_16);
         if (rc == -1)
         {
            slave_erro ++;
            std::cerr << "erro: " << modbus_strerror(errno) << std::endl;
         }
         else
         {
            slave_ok ++;
            for (int i = 0; i < 10; i++)
            {
               printf("red16[%d] = %d(0x%x)\n", i, read_buffer_16[i], read_buffer_16[i]);
            }
         }
         std::cout <<"send:"<<cyc_cont*8<<" read_ok:" << slave_ok*25 << " erro:" << slave_erro << std::endl;
         break;
      }
      case 2:
      {
         std::cout << "-----------从机模式----------" << std::endl;
         for (int i = 0; i < 10; i++)
         {
            mb_mapping->tab_registers[i] = data++;
         }
         if (data > 0XEEEE)
            data = 0;

         uint8_t query[MODBUS_TCP_MAX_ADU_LENGTH];
         if (modbusContext)
         {
            rc = modbus_receive(modbusContext, query);
            if (rc > 0)
            {
               poll_ok ++;
               modbus_reply(modbusContext, query, rc, mb_mapping);
            }
            else if (rc == -1)
            {
               poll_erro ++;
               std::cerr << "erro: " << modbus_strerror(errno) << std::endl;
            }
         }
         std::cout <<"send:"<<cyc_cont*25<< " read_ok:" << poll_ok*8 << " erro:" << poll_erro << std::endl;
         break;
      }

      default:
         break;
      }
   }
   modbus_mapping_free(mb_mapping);

   // 关闭 Modbus 连接
   modbus_close(modbusContext);
   modbus_free(modbusContext);

   return 0;
}
